package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date: 2021/4/20.
 */
@Service
public class ClientServiceImpl implements ClientService {

    @Autowired
    private ExamTaskService examTaskService;

    @Autowired
    private ExamTaskDetailService examTaskDetailService;

    @Autowired
    private ClientStatusService clientStatusService;

    @Autowired
    private ExamDetailService examDetailService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Autowired
    private BasicAttachmentService basicAttachmentService;

    @Autowired
    private CommonService commonService;

    @Autowired
    private ExamPrintPlanService examPrintPlanService;

    @Autowired
    private ExamCardDetailService examCardDetailService;

    @Override
    public IPage<ClientExamTaskDto> listTryTask(Long schoolId, String machineCode, Long orgId, Long printPlanId, String courseCode, String paperNumber, Boolean isTry, Boolean isPass, Integer pageNumber, Integer pageSize) {
        return examTaskService.listTryTask(schoolId, machineCode, orgId, printPlanId, courseCode, paperNumber, isTry, isPass, pageNumber, pageSize);
    }

    @Override
    public String getUrl(Long schoolId, Long examTaskId) {
        return examTaskDetailService.getUrl(schoolId, examTaskId);
    }

    @Override
    public Boolean tagPass(Long schoolId, String courseCode, String courseName, String paperNumber, String machineCode, Boolean isPass, Long userId) {
        return clientStatusService.tagPass(schoolId, courseCode, courseName, paperNumber, machineCode, isPass, userId);
    }

    @Override
    public Boolean updatePrintProgress(Long schoolId, Long examDetailId, Integer printProgress) {
        return examDetailService.updatePrintProgress(schoolId, examDetailId, printProgress);
    }

    @Override
    public IPage<ClientExamStudentDto> listStudent(Long schoolId, Long examDetailId, String ticketNumber, String studentName, String courseCode, Integer pageNumber, Integer pageSize) {
        return examDetailService.listStudent(schoolId, examDetailId, ticketNumber, studentName, courseCode, pageNumber, pageSize);
    }


    @Override
    public Map<String, Object> getReprintData(Long schoolId, Long examDetailId, String ticketNumber, String type) {
        Map<String, Object> finalMap = new HashMap<>();
        // 取试卷
        List<Map<String, String>> examDetailCourses = examDetailCourseService.listByExamDetailId(examDetailId);
        Map<String, String> map = mapCourseUrl(examDetailCourses);
        // 取考生
        List<Map<String, String>> studentList = examDetailService.listStudentByExamDetailId(schoolId, examDetailId, ticketNumber, type);
        // 生成试卷List
        List<ClientPrintDataDto> paperList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String paperNumber = m.get("paperNumber");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);

            StringJoiner sj = new StringJoiner(SystemConstant.DELIMITER);
            String key = sj.add(courseCode).add(paperNumber).add(paperType).toString();
            printDataDto.setUrl(map.get(key));
            return printDataDto;
        }).collect(Collectors.toList());
        finalMap.put("paper", paperList);
        // 生成题卡List
        List<ClientPrintDataDto> cardList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            String attachmentId = m.get("attachmentId");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);
            Map<String, String> urlMap = commonService.filePreview(attachmentId, false);
            printDataDto.setUrl(urlMap.get("pathUrl"));
            return printDataDto;
        }).collect(Collectors.toList());
        finalMap.put("card", cardList);
        //
        ExamDetail examDetail = examDetailService.getById(examDetailId);
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(examDetail.getPrintPlanId());
        String variableContent = examPrintPlan.getVariableContent();
        String ordinaryContent = examPrintPlan.getOrdinaryContent();
        List<Map> variableList = JSONObject.parseArray(variableContent, Map.class);
        List<Map> otherList = new ArrayList<>();
        for (Map variable : variableList) {
            Map vMap = new HashMap();
            vMap.put("type", variable.get("type"));
            Map<String, String> urlMap = commonService.filePreview(variable.get("attachmentId").toString(), false);
            vMap.put("htmlUrl", urlMap.get("htmlUrl"));
            vMap.put("pdfUrl", urlMap.get("pdfUrl"));
            otherList.add(vMap);
        }
        List<Map> ordinaryList = JSONObject.parseArray(ordinaryContent, Map.class);
        for (Map ordinary : ordinaryList) {
            Map vMap = new HashMap();
            vMap.put("type", ordinary.get("type"));
            Map<String, String> urlMap = commonService.filePreview(ordinary.get("attachmentId").toString(), false);
            vMap.put("htmlUrl", urlMap.get("htmlUrl"));
            vMap.put("pdfUrl", urlMap.get("pdfUrl"));
            otherList.add(vMap);
        }
        finalMap.put("other", otherList);
        return finalMap;
    }

    @Override
    public IPage<ClientPrintTaskDto> listClientPrintTask(Long schoolId, Long machineCode, String orgId, String printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Boolean isDownload, Boolean validate, Integer pageNumber, Integer pageSize) {
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        Page<ClientPrintTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ClientPrintTaskDto> pirntTaskDtoIPage = examPrintPlanService.listClientPrintTask(page, schoolId, machineCode, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime, isDownload, validate, orgIds);
        return pirntTaskDtoIPage;
    }

    @Override
    public Map<String, Object> getPrintData(Long schoolId, Long examDetailId, String machineCode, String printUser) {
        List<Map<String, String>> examDetailCourses = examDetailCourseService.listByExamDetailId(examDetailId);
        Map<String, Object> finalMap = new HashMap<>();
        // 2.取生成的完整的pdf
        ExamDetail examDetail = examDetailService.getById(examDetailId);
        if (examDetail.getAttachmentId() == null) {
            throw ExceptionResultEnum.ERROR.exception("考场pdf未生成");
        }

        BasicAttachment attachment = basicAttachmentService.getById(examDetail.getAttachmentId());
        if (attachment == null) {
            throw ExceptionResultEnum.ERROR.exception("考场pdf文件记录异常");
        }

        Map<String, String> attachMap = commonService.filePreview(examDetail.getAttachmentId().toString(), false);
        String totalPathUrl = attachMap.get("pathUrl");
        if (StringUtils.isNotBlank(totalPathUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考场pdf文件丢失");
        }

        finalMap.put("total", totalPathUrl);

        // 3.取学生试卷、题卡、备用试卷、题卡，印品
        Map<String, Object> detailMap = new HashMap<>();
        // 3.1取试卷
        Map<String, String> map = mapCourseUrl(examDetailCourses);
        // 3.2取考生
        List<Map<String, String>> studentList = examDetailService.listStudentByExamDetailId(schoolId, examDetailId, "1", "1");
        // 3.3生成试卷List
        List<ClientPrintDataDto> paperList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String paperNumber = m.get("paperNumber");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);

            StringJoiner sj = new StringJoiner(SystemConstant.DELIMITER);
            String key = sj.add(courseCode).add(paperNumber).add(paperType).toString();
            printDataDto.setUrl(map.get(key));
            return printDataDto;
        }).collect(Collectors.toList());
        detailMap.put("paper", paperList);
        // 3.4生成题卡List
        List<ClientPrintDataDto> cardList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            String attachmentId = m.get("attachmentId");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);
            Map<String, String> urlMap = commonService.filePreview(attachmentId, false);
            printDataDto.setUrl(urlMap.get("pathUrl"));
            return printDataDto;
        }).collect(Collectors.toList());
        detailMap.put("card", cardList);

        //3.4备用试卷、题卡
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(examDetail.getPrintPlanId());
        Map<String, Object> backupMap = new HashMap<>();
        int backupCount = examPrintPlan.getBackupCount().intValue();
        //试卷
        List<Map<String, String>> keyMaps = studentList.stream().map(m -> {
            Map<String, String> stringMap = new HashMap<>();
            stringMap.put("courseCode", m.get("courseCode"));
            stringMap.put("courseName", m.get("courseName"));
            stringMap.put("paperNumber", m.get("paperNumber"));
            stringMap.put("paperType", m.get("paperType"));
            return stringMap;
        }).distinct().collect(Collectors.toList());
        List<ClientPrintBackupDataDto> paperBackupList = new ArrayList<>();
        for (Map<String, String> keyMap : keyMaps) {
            for (int i = 0; i < backupCount; i++) {
                ClientPrintBackupDataDto clientPrintBackupDataDto = new ClientPrintBackupDataDto();
                clientPrintBackupDataDto.setCourseCode(keyMap.get("courseCode"));
                clientPrintBackupDataDto.setCourseName(keyMap.get("courseName"));
                clientPrintBackupDataDto.setPaperNumber(keyMap.get("paperNumber"));
                clientPrintBackupDataDto.setPaperType(keyMap.get("paperType"));
                StringJoiner sj = new StringJoiner(SystemConstant.DELIMITER);
                String key = sj.add(keyMap.get("courseCode")).add(keyMap.get("paperNumber")).add(keyMap.get("paperType")).toString();
                clientPrintBackupDataDto.setUrl(map.get(key));
                paperBackupList.add(clientPrintBackupDataDto);
            }
        }

        backupMap.put("paper", paperBackupList);

        // 备用题卡
        List<ClientPrintBackupDataDto> cardBackupList = new ArrayList<>();
        for (Map<String, String> examDetailCours : examDetailCourses) {
            String attachmentIds = examDetailCours.get("attachmentIds");
            JSONObject jsonObject = JSONObject.parseObject(attachmentIds);
            List<Map> backupCards = JSONObject.parseArray(JSONObject.toJSONString(jsonObject.get("card")), Map.class);
            for (Map backupCard : backupCards) {
                ClientPrintBackupDataDto clientPrintBackupDataDto = new ClientPrintBackupDataDto();
                clientPrintBackupDataDto.setCourseCode(backupCard.get("courseCode").toString());
                clientPrintBackupDataDto.setCourseName(backupCard.get("courseName").toString());
                clientPrintBackupDataDto.setPaperNumber(backupCard.get("paperNumber").toString());
                clientPrintBackupDataDto.setPaperType(backupCard.get("name").toString());
                Map<String, String> urlMap = commonService.filePreview(backupCard.get("attachmentId").toString(), false);
                clientPrintBackupDataDto.setUrl(urlMap.get("pathUrl"));
                cardBackupList.add(clientPrintBackupDataDto);
            }
        }
        backupMap.put("card", cardBackupList);

        //3.5
        String variableContent = examPrintPlan.getVariableContent();
        String ordinaryContent = examPrintPlan.getOrdinaryContent();
        List<Map> variableList = JSONObject.parseArray(variableContent, Map.class);
        List<Map> otherList = new ArrayList<>();
        for (Map variable : variableList) {
            int count = Integer.parseInt(variable.get("backupCount").toString());
            for (int i = 0; i < count; i++) {
                Map vMap = new HashMap();
                vMap.put("type", variable.get("type"));
                Map<String, String> urlMap = commonService.filePreview(variable.get("attachmentId").toString(), false);
                vMap.put("htmlUrl", urlMap.get("htmlUrl"));
                vMap.put("pdfUrl", urlMap.get("pdfUrl"));
                otherList.add(vMap);
            }
        }
        List<Map> ordinaryList = JSONObject.parseArray(ordinaryContent, Map.class);
        for (Map ordinary : ordinaryList) {
            int count = Integer.parseInt(ordinary.get("backupCount").toString());
            for (int i = 0; i < count; i++) {
                Map vMap = new HashMap();
                vMap.put("type", ordinary.get("type"));
                Map<String, String> urlMap = commonService.filePreview(ordinary.get("attachmentId").toString(), false);
                vMap.put("htmlUrl", urlMap.get("htmlUrl"));
                vMap.put("pdfUrl", urlMap.get("pdfUrl"));
                otherList.add(vMap);
            }

        }
        finalMap.put("other", otherList);
        return finalMap;
    }

    @Override
    public List<Map<String, Object>> getPrintDataBatch(Long schoolId, Long machineCode, String orgId, String printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Boolean isDownload, Boolean validate) {
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        List<ClientPrintTaskDto> pirntTaskDtoList = examPrintPlanService.listClientPrintTask(schoolId, machineCode, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime, isDownload, validate, orgIds);
        List<Map<String, Object>> finalList = new ArrayList<>();
        for (ClientPrintTaskDto clientPrintTaskDto : pirntTaskDtoList) {
            Map<String, Object> map = getPrintData(schoolId, Long.valueOf(clientPrintTaskDto.getExamDetailId()), null, null);
            finalList.add(map);
        }
        return finalList;
    }

    @Override
    public String getUrlByExamDetailId(Long schoolId, Long examDetailId) {
        ExamDetail examDetail = examDetailService.getById(examDetailId);
        BasicAttachment attachment = basicAttachmentService.getById(examDetail.getAttachmentId());
        return attachment == null ? null : commonService.filePreview(attachment.getPath());
    }

    @Override
    public Boolean updateDownload(Long schoolId, Long examDetailId, String machineCode, Boolean isDownload) {
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, schoolId).eq(ExamDetailCourse::getExamDetailId, examDetailId);
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(examDetailCourseQueryWrapper);
        for (ExamDetailCourse examDetailCours : examDetailCourses) {
            QueryWrapper<ClientStatus> clientStatusQueryWrapper = new QueryWrapper<>();
            clientStatusQueryWrapper.lambda().eq(ClientStatus::getSchoolId, schoolId).eq(ClientStatus::getCourseCode, examDetailCours.getCourseCode()).eq(ClientStatus::getPaperNumber, examDetailCours.getPaperNumber()).eq(ClientStatus::getMachineCode, machineCode);
            ClientStatus clientStatus = clientStatusService.getOne(clientStatusQueryWrapper);
            if (clientStatus == null) {
                clientStatus = new ClientStatus();
                clientStatus.setId(SystemConstant.getDbUuid());
                clientStatus.setSchoolId(schoolId);
                clientStatus.setMachineCode(machineCode);
                clientStatus.setCourseCode(examDetailCours.getCourseCode());
                clientStatus.setCourseName(examDetailCours.getCourseName());
                clientStatus.setPaperNumber(examDetailCours.getPaperNumber());
                clientStatus.setTry(false);
                clientStatus.setPass(false);
                clientStatus.setDownload(isDownload);
                clientStatusService.save(clientStatus);
            } else {
                clientStatus.setDownload(isDownload);
                clientStatusService.updateById(clientStatus);
            }
        }
        return true;
    }

    @Override
    public Boolean validateData(Long schoolId, Long examDetailId, String packageCode, String lastCode) {
        ExamDetail examDetail = examDetailService.getById(examDetailId);
        if (examDetail == null) {
            throw ExceptionResultEnum.ERROR.exception("考场数据有误");
        }
        if (StringUtils.isBlank(packageCode)) {
            throw ExceptionResultEnum.ERROR.exception("卷袋编号不能为空");
        }
        if (!packageCode.equals(examDetail.getPackageCode())) {
            throw ExceptionResultEnum.ERROR.exception("卷袋编号与系统数据不一致");
        }
        // 取最后一张备卡
        QueryWrapper<ExamDetailCourse> courseQueryWrapper = new QueryWrapper<>();
        courseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetailId).orderByDesc(ExamDetailCourse::getPaperNumber);
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(courseQueryWrapper);
        if (examDetailCourses.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("考场下课程数据异常");
        }
        ExamDetailCourse examDetailCourse = examDetailCourses.get(0);

        QueryWrapper<ExamTask> taskQueryWrapper = new QueryWrapper<>();
        taskQueryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode()).eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber());
        ExamTask examTask = examTaskService.getOne(taskQueryWrapper);
        if (examTask == null) {
            throw ExceptionResultEnum.ERROR.exception("命题任务数据异常");
        }

        QueryWrapper<ExamTaskDetail> taskDetailQueryWrapper = new QueryWrapper<>();
        taskDetailQueryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTask.getId());
        ExamTaskDetail examTaskDetail = examTaskDetailService.getOne(taskDetailQueryWrapper);
        if (examTaskDetail == null) {
            throw ExceptionResultEnum.ERROR.exception("命题任务数据异常");
        }
        ExamCardDetail examCardDetail = examCardDetailService.getByCardId(examTaskDetail.getCardId());
        if (examCardDetail == null) {
            throw ExceptionResultEnum.ERROR.exception("题卡数据异常");
        }
        String attachmentIds = examCardDetail.getAttachmentId();
        if (StringUtils.isBlank(attachmentIds)) {
            throw ExceptionResultEnum.ERROR.exception("备卡数据异常");
        }
        JSONObject jsonObject = JSONObject.parseObject(attachmentIds);
        String backupAttachmentIds = JSONObject.toJSONString(jsonObject.get("card"));
        List<Map> list = JSONObject.parseArray(backupAttachmentIds, Map.class);
        Map map = list.get(list.size() - 1);
        String backupPackageCode = map.get("packageCode").toString();
        if (StringUtils.isBlank(lastCode)) {
            throw ExceptionResultEnum.ERROR.exception("最后一张备卡数据异常");
        }
        if (!lastCode.equals(backupPackageCode)) {
            throw ExceptionResultEnum.ERROR.exception("最后一张备卡条码与系统数据不一致");
        }

        String lastCodeSub = lastCode.substring(0, lastCode.length() - 2);
        if (packageCode.equals(lastCodeSub)) {
            throw ExceptionResultEnum.ERROR.exception("最后一张备卡条码与试卷编号不一致");
        }
        return true;
    }

    /**
     * 根据考场生成试卷map
     *
     * @return
     */
    public Map<String, String> mapCourseUrl(List<Map<String, String>> mapList) {
        Map<String, String> map = new HashMap<>();
        for (Map<String, String> taskDetail : mapList) {
            String courseCode = taskDetail.get("courseCode");
            String paperNumber = taskDetail.get("paperNumber");
            String paperAttachmentIds = taskDetail.get("paperAttachmentIds");
            List<Map> attachementIds = JSONObject.parseArray(paperAttachmentIds, Map.class);
            for (Map attaMap : attachementIds) {
                String name = attaMap.get("name").toString();
                String attachmentId = attaMap.get("attachmentId").toString();
                BasicAttachment attachment = basicAttachmentService.getById(attachmentId);
                String url = commonService.filePreview(attachment.getPath());
                StringJoiner sj = new StringJoiner(SystemConstant.DELIMITER);
                String key = sj.add(courseCode).add(paperNumber).add(name).toString();
                map.put(key, url);
            }
        }
        return map;
    }
}
